// ##########################################################################
// ####                                                                  ####
// ####                    Master Thesis Implementation                  ####
// ####  Management of Layered Variable Bitrate Multimedia Streams over  ####
// ####                  DiffServ with A Priori Knowledge                ####
// ####                                                                  ####
// #### ================================================================ ####
// ####                                                                  ####
// ####                                                                  ####
// #### Global Trace Configuration                                       ####
// ####                                                                  ####
// #### Version 1.00  --  October 10, 2000                               ####
// ####                                                                  ####
// #### Copyright (C) 2000 Thomas Dreibholz                              ####
// #### University of Bonn, Department of Computer Science IV            ####
// #### EMail: Dreibholz@bigfoot.com                                     ####
// #### WWW:   http://www.bigfoot.com/~dreibholz/diplom/index.html       ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "globaltraceconfiguration.h"


namespace Coral {


// ###### Configuration defaults ############################################
TraceConfiguration TraceConfig = {
   25.0,                                       // FrameRate
   "IBBPBBPBBPBB",                             // FramePattern
   "",                                         // InputName

   0,                                          // ExtLayers
   0.0,                                        // FakeE1
   0.0,                                        // FakeE2

   500000,                                     // MaxBufferDelay [s]
   0,                                          // MinIntervalLength [s]
   (card64)-1,                                 // MaxIntervalLength [s]
   3000000.0,                                  // RemappingCost


   64,                                         // Resource/Utilization points
   10000,                                      // BandwidthThreshold
   0.005,                                      // UtilizationThreshold

   4,                                          // FrameRateUtilizationMaxConstants
   0,                                          // FrameRateUtilizationConstants
   UF_Linear,                                  // FrameRateUtilizationType
   1.0,                                        // FrameRateUtilizationWeight
   {0.0, 0.0, 0.0, 0.0,                        // FrameRateUtilization
    0.0, 0.0, 0.0, 0.0},


   {""},                                       // Title
   {""},                                       // Copyright
   {""},                                       // Comment
   {""},                                       // URL

   {
      {
         0.5,                                  // Scalability
         4.0,                                  // CostFactor
         8,                                    // ByterateEmpiricalEnvelopePairs
         4,                                    // FramecountEmpiricalEnvelopePairs
         0x00,                                 // Layer flags
         4,                                    // FrameSizeUtilizationMaxConstants
         0,                                    // FrameSizeUtilizationConstants
         UF_Linear,                            // FrameSizeUtilizationType
         1.0,                                  // FrameSiteUtilizationWeight
         {0.0, 0.0, 0.0, 0.0,                  // FrameSizeUtilization
          0.0, 0.0, 0.0, 0.0}
      },
      {
         0.5,                                  // Scalability
         3.0,                                  // CostFactor
         12,                                   // ByterateEmpiricalEnvelopePairs
         4,                                    // FramecountEmpiricalEnvelopePairs
         0x00,                                 // Layer flags
         4,                                    // FrameSizeUtilizationMaxConstants
         0,                                    // FrameSizeUtilizationConstants
         UF_Linear,                            // FrameSizeUtilizationType
         1.0,                                  // FrameSiteUtilizationWeight
         {0.0, 0.0, 0.0, 0.0,                  // FrameSizeUtilization
          0.0, 0.0, 0.0, 0.0}
      },
      {
         0.5,                                  // Scalability
         2.0,                                  // CostFactor
         12,                                   // ByterateEmpiricalEnvelopePairs
         4,                                    // FramecountEmpiricalEnvelopePairs
         0x00,                                 // Layer flags
         4,                                    // FrameSizeUtilizationMaxConstants
         0,                                    // FrameSizeUtilizationConstants
         UF_Linear,                            // FrameSizeUtilizationType
         1.0,                                  // FrameSiteUtilizationWeight
         {0.0, 0.0, 0.0, 0.0,                  // FrameSizeUtilization
          0.0, 0.0, 0.0, 0.0}
      },
      {
         0.5,                                  // Scalability
         1.5,                                  // CostFactor
         12,                                   // ByterateEmpiricalEnvelopePairs
         4,                                    // FramecountEmpiricalEnvelopePairs
         0x00,                                 // Layer flags
         4,                                    // FrameSizeUtilizationMaxConstants
         0,                                    // FrameSizeUtilizationConstants
         UF_Linear,                            // FrameSizeUtilizationType
         1.0,                                  // FrameSiteUtilizationWeight
         {0.0, 0.0, 0.0, 0.0,                  // FrameSizeUtilization
          0.0, 0.0, 0.0, 0.0}
      },
      {
         0.5,                                  // Scalability
         1.3,                                  // CostFactor
         12,                                   // ByterateEmpiricalEnvelopePairs
         4,                                    // FramecountEmpiricalEnvelopePairs
         0x00,                                 // Layer flags
         4,                                    // FrameSizeUtilizationMaxConstants
         0,                                    // FrameSizeUtilizationConstants
         UF_Linear,                            // FrameSizeUtilizationType
         1.0,                                  // FrameSiteUtilizationWeight
         {0.0, 0.0, 0.0, 0.0,                  // FrameSizeUtilization
          0.0, 0.0, 0.0, 0.0}
      },
      {
         0.5,                                  // Scalability
         1.2,                                  // CostFactor
         12,                                   // ByterateEmpiricalEnvelopePairs
         4,                                    // FramecountEmpiricalEnvelopePairs
         0x00,                                 // Layer flags
         4,                                    // FrameSizeUtilizationMaxConstants
         0,                                    // FrameSizeUtilizationConstants
         UF_Linear,                            // FrameSizeUtilizationType
         1.0,                                  // FrameSiteUtilizationWeight
         {0.0, 0.0, 0.0, 0.0,                  // FrameSizeUtilization
          0.0, 0.0, 0.0, 0.0}
      },
      {
         0.5,                                  // Scalability
         1.1,                                  // CostFactor
         12,                                   // ByterateEmpiricalEnvelopePairs
         4,                                    // FramecountEmpiricalEnvelopePairs
         0x00,                                 // Layer flags
         4,                                    // FrameSizeUtilizationMaxConstants
         0,                                    // FrameSizeUtilizationConstants
         UF_Linear,                            // FrameSizeUtilizationType
         1.0,                                  // FrameSiteUtilizationWeight
         {0.0, 0.0, 0.0, 0.0,                  // FrameSizeUtilization
          0.0, 0.0, 0.0, 0.0}
      },
      {
         0.5,                                  // Scalability
         1.0,                                  // CostFactor
         12,                                   // ByterateEmpiricalEnvelopePairs
         4,                                    // FramecountEmpiricalEnvelopePairs
         0x00,                                 // Layer flags
         4,                                    // FrameSizeUtilizationMaxConstants
         0,                                    // FrameSizeUtilizationConstants
         UF_Linear,                            // FrameSizeUtilizationType
         1.0,                                  // FrameSiteUtilizationWeight
         {0.0, 0.0, 0.0, 0.0,                  // FrameSizeUtilization
          0.0, 0.0, 0.0, 0.0}
      },
      {
         0.5,                                  // Scalability
         1.0,                                  // CostFactor
         12,                                   // ByterateEmpiricalEnvelopePairs
         4,                                    // FramecountEmpiricalEnvelopePairs
         0x00,                                 // Layer flags
         4,                                    // FrameSizeUtilizationMaxConstants
         0,                                    // FrameSizeUtilizationConstants
         UF_Linear,                            // FrameSizeUtilizationType
         1.0,                                  // FrameSiteUtilizationWeight
         {0.0, 0.0, 0.0, 0.0,                  // FrameSizeUtilization
          0.0, 0.0, 0.0, 0.0}
      }
   }
};


}
