// ##########################################################################
// ####                                                                  ####
// ####                    Master Thesis Implementation                  ####
// ####  Management of Layered Variable Bitrate Multimedia Streams over  ####
// ####                  DiffServ with A Priori Knowledge                ####
// ####                                                                  ####
// #### ================================================================ ####
// ####                                                                  ####
// ####                                                                  ####
// #### MP3 QoS Description Implementation                               ####
// ####                                                                  ####
// #### Version 1.00  --  October 10, 2000                               ####
// ####                                                                  ####
// #### Copyright (C) 2000 Thomas Dreibholz                              ####
// #### University of Bonn, Department of Computer Science IV            ####
// #### EMail: Dreibholz@bigfoot.com                                     ####
// #### WWW:   http://www.bigfoot.com/~dreibholz/diplom/index.html       ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "mp3qosdescription.h"


namespace Coral {


// ###### Calculate MP3's maximum utilization for given total bandwidths ###
void MP3QoSDescription::calculateMaxUtilizationForBandwidthArray(
                                  const cardinal*           totalBandwidthArray,
                                  ResourceUtilizationPoint* rupArray,
                                  const cardinal            points) const
{
   // ====== Initialize =====================================================
   bool  finished[points];
   for(cardinal i = 0;i < points;i++) {
      rupArray[i].Layers      = 1;
      rupArray[i].Utilization = -1.0;
      rupArray[i].Cost        = 0.0;
      rupArray[i].TaskID      = 0;
      rupArray[i].Bandwidth   = 0;
      for(cardinal j = 0;j < 1;j++) {
         rupArray[i].LayerBandwidthInfo[j].BufferDelay      = Layer[j].getBufferDelay();
         rupArray[i].LayerBandwidthInfo[j].MaxTransferDelay = Layer[j].getMaxTransferDelay();
         rupArray[i].LayerBandwidthInfo[j].MaxLossRate      = Layer[j].getMaxLossRate();
         rupArray[i].LayerBandwidthInfo[j].MaxJitter        = Layer[j].getMaxJitter();
      }
      finished[i] = false;
   }

   const double frameRate = getMinFrameRate();
   const cardinal maxBandwidth = AbstractLayerDescription::frameSizeToBandwidth(
                                    frameRate, Layer[0].getMaxFrameSize(frameRate));
   const cardinal minBandwidth = AbstractLayerDescription::frameSizeToBandwidth(
                                    frameRate, Layer[0].getMinFrameSize(frameRate));

   cardinal bandwidth[1];
   for(cardinal i = 0;i < points;i++) {
      if(totalBandwidthArray[i] >= maxBandwidth) {
         bandwidth[0] = maxBandwidth;
      }
      else if(totalBandwidthArray[i] < minBandwidth) {
         bandwidth[0] = 0;
      }
      else {
         bandwidth[0] = totalBandwidthArray[i];
      }
      rupArray[i].Utilization = calculateUtilizationForLayerBandwidths(
                                   frameRate, 1,
                                   (cardinal*)&bandwidth[0]);
      rupArray[i].FrameRate   = frameRate;
      rupArray[i].Bandwidth   = bandwidth[0];
      rupArray[i].LayerBandwidthInfo[0].BytesPerSecond   = bandwidth[0];
      rupArray[i].LayerBandwidthInfo[0].PacketsPerSecond =
         Layer[0].frameSizeToPacketRate(
            rupArray[i].FrameRate,
            Layer[0].bandwidthToFrameSize(rupArray[i].FrameRate,bandwidth[0]));
   }
}


}
