// ##########################################################################
// ####                                                                  ####
// ####                    Master Thesis Implementation                  ####
// ####  Management of Layered Variable Bitrate Multimedia Streams over  ####
// ####                  DiffServ with A Priori Knowledge                ####
// ####                                                                  ####
// #### ================================================================ ####
// ####                                                                  ####
// ####                                                                  ####
// #### Stream Description Implementation                                ####
// ####                                                                  ####
// #### Version 1.00  --  October 10, 2000                               ####
// ####                                                                  ####
// #### Copyright (C) 2000 Thomas Dreibholz                              ####
// #### University of Bonn, Department of Computer Science IV            ####
// #### EMail: Dreibholz@bigfoot.com                                     ####
// #### WWW:   http://www.bigfoot.com/~dreibholz/diplom/index.html       ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "streamdescription.h"


namespace Coral {


// ###### Constructor #######################################################
StreamDescription::StreamDescription()
{
}


// ###### Destructor ########################################################
StreamDescription::~StreamDescription()
{
}


// ###### Initialize StreamDescription ######################################
void StreamDescription::init(ManagedStreamInterface* stream)
{
   Interface      = stream;
   if(QoSDescription != NULL) {
      delete QoSDescription;
   }
   QoSDescription = stream->getQoSDescription(0);
   Layers         = QoSDescription->getLayers();
   OldUtilization = -HUGE_VAL;
   NewUtilization = -HUGE_VAL;
   MaximumReached = false;



   const cardinal maxPoints = 32;
   const cardinal bwThreshold = 10000;
   const double   utThreshold = 0.10;



   ResourceUtilizationPoint RUList1[maxPoints];

//   stream->lock();
   cardinal RUEntries1 = QoSDescription->getPrecomputedResourceUtilizationList(
                            (ResourceUtilizationPoint*)&RUList1,
                            bwThreshold, utThreshold, maxPoints);
//   stream->unlock();

   RUEntries = RUEntries1;
   for(cardinal i = 0;i <RUEntries1;i++) {
      RUList[i] = RUList1[i];
   }
}


// ###### Calculate possible mappings from layers to DiffServ classes #######
bool StreamDescription::calculateLayerClassMapping(
                           const ServiceLevelAgreement*    sla,
                           const ResourceUtilizationPoint* rup)
{
   for(cardinal i = 0;i < Layers;i++) {
      LayerClasses[i] =
         sla->getPossibleClassesForBandwidthInfo(rup->LayerBandwidthInfo[i],
                                                 (cardinal*)&LayerClassMapping[i]);
      if((LayerClasses[i] == 0) &&
         (rup->LayerBandwidthInfo[i].BytesPerSecond > 0)) {
         return(false);
      }
   }
   return(true);
}


// ###### Try to allocate given bandwidths to stream's layers ###############
bool StreamDescription::tryAllocation(
                           cardinal&                 totalAvailableBandwidth,
                           cardinal*                 classAvailableBandwidthArray,
                           ResourceUtilizationPoint* rup)
{
   // ====== Backup old values ==============================================
   cardinal oldTotalAvailableBandwidth = totalAvailableBandwidth;
   cardinal oldClassAvailableBandwidthArray[TrafficClassValues::MaxValues];
   for(cardinal i = 0;i < TrafficClassValues::MaxValues;i++) {
      oldClassAvailableBandwidthArray[i] = classAvailableBandwidthArray[i];
   }


   // ====== Free old allocation ============================================
   for(cardinal i = 0;i < Layers;i++) {
      totalAvailableBandwidth += ClassBandwidth[i];
      classAvailableBandwidthArray[ClassNumber[i]] += ClassBandwidth[i];
   }


   // ====== Check, if enough bandwidth is available ========================
   if(totalAvailableBandwidth >= rup->Bandwidth) {

      // ====== Get bandwidth for all layers ================================
      for(cardinal i = 0;i < Layers;i++) {

         // ====== Try to get bandwidth from cheapest possible class ========
         bool mappedLayer = false;
         for(cardinal j = 0;j < LayerClasses[i];j++) {
            const cardinal classNumber = LayerClassMapping[i][j];
            if(classAvailableBandwidthArray[classNumber] >= rup->LayerBandwidthInfo[i].BytesPerSecond) {
               classAvailableBandwidthArray[classNumber] -= rup->LayerBandwidthInfo[i].BytesPerSecond;
               totalAvailableBandwidth -= rup->LayerBandwidthInfo[i].BytesPerSecond;
               ClassNumber[i]    = classNumber;
               ClassBandwidth[i] = rup->LayerBandwidthInfo[i].BytesPerSecond;
               mappedLayer = true;
               break;
            }
         }
         if(mappedLayer == false) {
            goto rollback;
         }
      }

      // ====== Success =====================================================
      return(true);
   }


   // ====== Allocation failed ==============================================
rollback:
   for(cardinal i = 0;i < TrafficClassValues::MaxValues;i++) {
      classAvailableBandwidthArray[i] = oldClassAvailableBandwidthArray[i];
   }
   totalAvailableBandwidth = oldTotalAvailableBandwidth;
   return(false);
}


}
