#include "abstractlayerdescription.h"
#include "abstractqosdescription.h"
#include "rtppacket.h"
#include "trafficclassvalues.h"
#include "tdtfmediareader.h"
#include "traceqosdescription.h"
#include "streamdescription.h"
#include "servicelevelagreement.h"


using namespace Coral;


ServiceLevelAgreement gSLA;
ServiceLevelAgreement* SLA = &gSLA;


int main()
{
   puts("START!");
   TDTFMediaReader tr;
   if(tr.open("xx.tdtf")) {
      tr.setFrameRate(25.0);
      tr.setPosition((card64)39960000000.0);
//      8818420609.0);

card64 oldPos = tr.getPosition();
      if(gSLA.load("SLA.config") == false) {
         puts("SLA load!");
         exit(1);
      }
      cout << gSLA;


cardinal totalAvailableBandwidth;
cardinal classAvailableBandwidthArray[TrafficClassValues::MaxValues];



      totalAvailableBandwidth = SLA->TotalBandwidth;
      for(cardinal i = 0;i < SLA->Classes;i++) {
         classAvailableBandwidthArray[i] = SLA->Class[i].BytesPerSecond;
      }



      const cardinal streams   = 1;
      const cardinal maxPoints = 32;
      const cardinal bwThreshold = 10000;
      const double   utThreshold = 0.10;


      TraceQoSDescription  qd[streams];
      TraceQoSDescription* qosDescription[streams];
      for(cardinal i = 0;i < streams;i++) {
        qosDescription[i] = &qd[i];
        qd[i].initTraceDescription(&tr);
        qd[i].updateDescription(20+8+12+26,1500);
      }


      StreamDescription sd[streams];
      StreamDescription* streamDescription[streams];
      for(cardinal i = 0;i < streams;i++) {
         streamDescription[i] = &sd[i];
         streamDescription[i]->init(qosDescription[i]);
      }


      card64 s = getMicroTime();


      // ====== Get resource/utilization list ===============================
      ResourceUtilizationPoint* rupList = new ResourceUtilizationPoint[streams * maxPoints];
      cardinal points = 0;
      for(cardinal i = 0;i < streams;i++) {
         const cardinal p = streamDescription[i]->QoSDescription->getPrecomputedResourceUtilizationList(
                               (ResourceUtilizationPoint*)&rupList[points],
                               bwThreshold, utThreshold, maxPoints);
         if(streamDescription[i]->calculateLayerClassMapping(SLA,&rupList[points]) == true) {
            for(cardinal j = 0;j < p;j++) {
               rupList[points + j].TaskID = i;
            }
            streamDescription[i]->MaximumReached = false;

            points += p;
         }
      }
/*
for(int i = 0;i < points;i++) {
   cout << rupList[i] << endl;
}
*/
      // ====== Sort resource/utilization list ==============================
      ResourceUtilizationPoint::sortResourceUtilizationList(rupList,0,points - 1);


      // ====== Get old utilization, reset description zu bandwidth 0 =======
      for(cardinal i = 0;i < streams;i++) {
         ResourceUtilizationPoint rup;
         streamDescription[i]->OldUtilization =
            streamDescription[i]->QoSDescription->getResources(rup);
         rup.Bandwidth = 0;
         for(cardinal j = 0;j < rup.Layers;j++) {
            rup.LayerBandwidthInfo[j].BytesPerSecond   = 0;
            rup.LayerBandwidthInfo[j].PacketsPerSecond = 0;
         }
         streamDescription[i]->QoSDescription->setResources(rup);
      }


      // ====== Allocate bandwidth ==========================================
puts("----------------------------------------------------");

printf("POS=%Ld  %d    FR=%f\n",oldPos,
tr.positionToFramePosition(tr.getFrameRate(),oldPos),
tr.getFrameRate());
/*
cardinal x = tr.positionToFramePosition(tr.getFrameRate(),oldPos);
card64 y = (card64)floor(((double)x * 1000000000.0) / tr.getFrameRate());
tr.setPosition(y);


printf("x=%d y=%Ld  diff=%Ld\n",x,y,(int64)oldPos - (int64)y);
*/



      for(cardinal i = 0;i < points;i++) {
         const cardinal taskID = rupList[i].TaskID;

         if(streamDescription[taskID]->MaximumReached == false) {
            const bool ok = streamDescription[taskID]->tryAllocation(
                               totalAvailableBandwidth,
                               (cardinal*)&classAvailableBandwidthArray,
                               &rupList[i]);
            if(ok == false) {
/*
if(taskID == 0) {
cout << "FAIL: " << rupList[i] << endl;
}
*/
               streamDescription[taskID]->MaximumReached = true;
            }
            else {
/*
cardinal x = tr.positionToFramePosition(rupList[i].FrameRate,oldPos);
card64   y = tr.framePositionToPosition(rupList[i].FrameRate,x);
card64   z = tr.framePositionToPosition(25.0,298);
printf("OLD=%Ld  NEW=%Ld  OPT=%Ld\n",oldPos,y,z);
tr.setPosition(y);
*/
//tr.setPosition(tr.framePositionToPosition(25.0,298));
//printf("OLD=%Ld  NEW=%Ld\n",oldPos,tr.getPosition());
// rup.Position bezieht sich auf das Intervall der RU-Liste,
// es ist daher die *korrekte* Position!!!!!!!!!!!!!!!!!!!!!!
/*
printf("rp=%d\n",rupList[i].Position);
tr.setPosition(tr.framePositionToPosition(rupList[i].FrameRate,rupList[i].Position));
printf("OLD=%d  NEW=%d\n",
tr.positionToFramePosition(rupList[i].FrameRate,oldPos),
rupList[i].Position);
card64 newPos = tr.getPosition();

// ---------------------------------------------------------------

cardinal o = tr.positionToFramePosition(tr.getFrameRate(),oldPos);

card64 c;
c = (card64)floor((rupList[i].FrameRate * (double)o) / tr.getFrameRate());

printf("O=%Ld  N=%Ld  C=%Ld\n",oldPos,newPos,c);
*/

               streamDescription[taskID]->NewUtilization =
                  streamDescription[taskID]->QoSDescription->setResources(rupList[i]);
               if(fabs(streamDescription[taskID]->NewUtilization - rupList[i].Utilization) > 0.01) {
                  cerr << "WARNING: Utilization from list and calculated value differ!" << endl
                       << "List Value = " << rupList[i] << endl
                       << "Calculated = " << streamDescription[taskID]->NewUtilization << endl;
               }
               else
                  cerr << "OK:          " << rupList[i] << endl;
            }
         }
      }

/*
      card64 e = getMicroTime();
      printf("pts=%d   t=%Ld\n",points,e-s);

      for(cardinal i = 0;i < streams;i++) {
          ResourceUtilizationPoint rup;
          streamDescription[i]->QoSDescription->getResources(rup);
          cout << i << ": " << rup << "  " << streamDescription[i]->OldUtilization << " vs. " << streamDescription[i]->NewUtilization << endl;
      }


      printf("TOTAL=%d\n",totalAvailableBandwidth);
      for(cardinal i = 0;i < SLA->Classes;i++) {
         printf("#%d: %7d\n",i,classAvailableBandwidthArray[i]);
      }
*/
   }
   return(0);
}
