// ##########################################################################
// ####                                                                  ####
// ####                    Master Thesis Implementation                  ####
// ####  Management of Layered Variable Bitrate Multimedia Streams over  ####
// ####                  DiffServ with A Priori Knowledge                ####
// ####                                                                  ####
// #### ================================================================ ####
// ####                                                                  ####
// ####                                                                  ####
// #### Trace Decoder Repository Implementation                          ####
// ####                                                                  ####
// #### Version 1.00  --  October 10, 2000                               ####
// ####                                                                  ####
// #### Copyright (C) 2000 Thomas Dreibholz                              ####
// #### University of Bonn, Department of Computer Science IV            ####
// #### EMail: Dreibholz@bigfoot.com                                     ####
// #### WWW:   http://www.bigfoot.com/~dreibholz/diplom/index.html       ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "tracedecoderrepository.h"


namespace Coral {


// ###### Constructor #######################################################
TraceDecoderRepository::TraceDecoderRepository()
{
   Decoder = NULL;
}


// ###### Destructor ########################################################
TraceDecoderRepository::~TraceDecoderRepository()
{
   if(Decoder) {
      Decoder->deactivate();
   }
   if(AutoDelete) {
      while(TraceDecoderRepository.begin() != TraceDecoderRepository.end()) {
         multimap<const card16,TraceDecoderInterface*>::iterator decoderIterator =
            TraceDecoderRepository.begin();
         Decoder = decoderIterator->second;
         TraceDecoderRepository.erase(decoderIterator);
         delete Decoder;
      }
   }
   Decoder = NULL;
}


// ###### Add decoder to repository #########################################
bool TraceDecoderRepository::addDecoder(TraceDecoderInterface* decoder)
{
   multimap<const card16,TraceDecoderInterface*>::iterator decoderIterator =
      TraceDecoderRepository.find(decoder->getTypeID());
   if(decoderIterator == TraceDecoderRepository.end()) {
      TraceDecoderRepository.insert(pair<const card16,TraceDecoderInterface*>
                                    (decoder->getTypeID(),decoder));
      if(Decoder == NULL) {
         Decoder = decoder;
      }
      return(true);
   }
   return(false);
}


// ###### Remove decoder from repository ####################################
void TraceDecoderRepository::removeDecoder(TraceDecoderInterface* decoder)
{
   multimap<const card16,TraceDecoderInterface*>::iterator decoderIterator =
      TraceDecoderRepository.find(decoder->getTypeID());
   if(decoderIterator != TraceDecoderRepository.end()) {
      TraceDecoderRepository.erase(decoderIterator);
      decoder->deactivate();
      if(Decoder == decoder) {
         multimap<const card16,TraceDecoderInterface*>::iterator firstDecoder =
            TraceDecoderRepository.begin();
         if(firstDecoder != TraceDecoderRepository.end()) {
            Decoder = firstDecoder->second;
         }
         else {
            Decoder = NULL;
         }
      }
      else {
         Decoder = NULL;
      }
   }
}


// ###### Select decoder for given type ID ##################################
bool TraceDecoderRepository::selectDecoderForTypeID(const card16 typeID)
{
   if((Decoder == NULL) || (typeID != Decoder->getTypeID())) {
      multimap<const card16,TraceDecoderInterface*>::iterator decoderIterator =
         TraceDecoderRepository.find(typeID);
      if(decoderIterator != TraceDecoderRepository.end()) {
          TraceDecoderInterface* decoder = decoderIterator->second;
          Decoder->deactivate();

          Decoder = decoder;

          Decoder->activate();
          return(true);
      }
      return(false);
   }
   return(true);
}


// ###### Get pointer to current decoder ####################################
DecoderInterface* TraceDecoderRepository::getCurrentDecoder() const
{
   return((DecoderInterface*)Decoder);
}


// ###### Get pointer to current trace decoder ##############################
TraceDecoderInterface* TraceDecoderRepository::getCurrentTraceDecoder() const
{
   return(Decoder);
}


/*
#############################################################################
###### State Pattern: Method calls                                     ######
#############################################################################
*/

const card16 TraceDecoderRepository::getTypeID() const
   { return(Decoder->getTypeID()); }

const char* TraceDecoderRepository::getTypeName() const
   { return(Decoder->getTypeName()); }

void TraceDecoderRepository::getMediaInfo(MediaInfo& mediaInfo) const
   { Decoder->getMediaInfo(mediaInfo); }

card8 TraceDecoderRepository::getErrorCode() const
   { return(Decoder->getErrorCode()); }

void TraceDecoderRepository::activate()
   { Decoder->activate(); }

void TraceDecoderRepository::deactivate()
   { Decoder->deactivate(); }

void TraceDecoderRepository::reset()
   { Decoder->reset(); }

bool TraceDecoderRepository::checkNextPacket(DecoderPacket* decoderPacket)
   { return(Decoder->checkNextPacket(decoderPacket)); }

void TraceDecoderRepository::handleNextPacket(const DecoderPacket* decoderPacket)
   { Decoder->handleNextPacket(decoderPacket); }

card64 TraceDecoderRepository::getPosition() const
   { return(Decoder->getPosition()); }

card64 TraceDecoderRepository::getMaxPosition() const
   { return(Decoder->getMaxPosition()); }

double TraceDecoderRepository::getFrameRate() const
   { return(Decoder->getFrameRate()); }

double TraceDecoderRepository::getUtilization() const
   { return(Decoder->getUtilization()); }

Range<cardinal> TraceDecoderRepository::getBandwidth() const
   { return(Decoder->getBandwidth()); }

int8 TraceDecoderRepository::getStreamPriority() const
   { return(Decoder->getStreamPriority()); }

int8 TraceDecoderRepository::getSessionPriority() const
   { return(Decoder->getSessionPriority()); }

}
