// ##########################################################################
// ####                                                                  ####
// ####                    Master Thesis Implementation                  ####
// ####  Management of Layered Variable Bitrate Multimedia Streams over  ####
// ####                  DiffServ with A Priori Knowledge                ####
// ####                                                                  ####
// #### ================================================================ ####
// ####                                                                  ####
// ####                                                                  ####
// #### MP3 Trace Array Implementation                                   ####
// ####                                                                  ####
// #### Version 1.00  --  October 10, 2000                               ####
// ####                                                                  ####
// #### Copyright (C) 2000 Thomas Dreibholz                              ####
// #### University of Bonn, Department of Computer Science IV            ####
// #### EMail: Dreibholz@bigfoot.com                                     ####
// #### WWW:   http://www.bigfoot.com/~dreibholz/diplom/index.html       ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "mp3tracearray.h"
#include "randomizer.h"


#include <fstream.h>


namespace Coral {


// ###### Constructor #######################################################
MP3TraceArray::MP3TraceArray(const TraceConfiguration& config)
   : TraceArray(config)
{
}


// ###### Load MP3 trace ###################################################
bool MP3TraceArray::load(const char* name)
{
   // ====== Open trace file, allocate memory ===============================
   ifstream is(name);
   if(is.good()) {
      // ====== Count frames ================================================
      cardinal frameCount = 0;
      cardinal id         = 0;
      cardinal frameSizeB = 0;
      cardinal idB        = 0;
      while(!is.eof()) {
         is >> frameSizeB;
         frameCount++;
      }
      is.close();
      is.open(name);

      // ====== Initialize ==================================================
      cardinal patternPos = 0;
      init(1,frameCount);
      FrameRate = Config.FrameRate;

      // ====== Read frame sizes ============================================
      for(;;) {
         if(Frames >= MaxFrames) {
            cerr << "ERROR: MP3TraceArray::load() - Too many frames!" << endl;
            return(false);
         }

         // ====== Read frame size ==========================================
         is >> frameSizeB;
         if(is.eof()) {
            break;
         }
         idB = id++;

         // ====== Write trace into array ===================================
         for(cardinal i = 1;i < MaxLayers;i++) {
            LayerTrace[i]->Frame[Frames].ID   = (cardinal)-1;
            LayerTrace[i]->Frame[Frames].Size = 0;
            LayerTrace[i]->Frames++;
         }
         LayerTrace[0]->Frame[Frames].ID   = idB;
         LayerTrace[0]->Frame[Frames].Size = frameSizeB;

         Frames++;
         patternPos++;
      }

      // ====== Ensure correct alignment ====================================
      const cardinal diff = Frames % (cardinal)rint(FrameRate);
      Frames -= diff;
      return(Frames > 0);
   }
   else {
      cerr << "ERROR: MP3TraceArray::load() - Unable to load trace " << name << "!" << endl;
      return(false);
   }
}


}
