// ##########################################################################
// ####                                                                  ####
// ####                    Master Thesis Implementation                  ####
// ####  Management of Layered Variable Bitrate Multimedia Streams over  ####
// ####                  DiffServ with A Priori Knowledge                ####
// ####                                                                  ####
// #### ================================================================ ####
// ####                                                                  ####
// ####                                                                  ####
// #### MP3 Writer QoS Description                                       ####
// ####                                                                  ####
// #### Version 1.00  --  October 10, 2000                               ####
// ####                                                                  ####
// #### Copyright (C) 2000 Thomas Dreibholz                              ####
// #### University of Bonn, Department of Computer Science IV            ####
// #### EMail: Dreibholz@bigfoot.com                                     ####
// #### WWW:   http://www.bigfoot.com/~dreibholz/diplom/index.html       ####
// ####                                                                  ####
// ##########################################################################


#ifndef MP3WRITERQOSDESCRIPTION_H
#define MP3WRITERQOSDESCRIPTION_H


#include "system.h"
#include "mp3qosdescription.h"
#include "traceconfiguration.h"


namespace Coral {


/**
  * This is the QoS description of a trace, extended by methods for
  * MP3 resource/utilization calculation. These overwrite the ones which
  * are used for calculating utilization from the trace file by the ones
  * which use a new configuration. This is necessary to update the
  * utilization of the file.
  *
  * @short MP3 Writer QoS Description
  * @author  Thomas Dreibholz (Dreibholz@bigfoot.com)
  * @version 1.0
  */
class MP3WriterQoSDescription : public MP3QoSDescription
{
   // ====== Constructor/Destructor =========================================
   public:
   /**
     * Constructor.
     *
     * @param traceConfiguration Configuration.
     */
   MP3WriterQoSDescription(const TraceConfiguration* traceConfiguration);


   // ====== Reimplementation of utilization methods ========================
   /**
     * Implementation of AbstractQoSDescription's calculateUtilizationForLayerBandwidths().
     *
     * @see AbstractQoSDescription#calculateUtilizationForLayerBandwidths
     */
   double calculateUtilizationForLayerBandwidths(const double    frameRate,
                                                 const cardinal  layers,
                                                 const cardinal* bandwidth) const;


   // ====== Private data ===================================================
   private:
   const TraceConfiguration* Config;
};


}


#endif
