// ##########################################################################
// ####                                                                  ####
// ####                    Master Thesis Implementation                  ####
// ####  Management of Layered Variable Bitrate Multimedia Streams over  ####
// ####                  DiffServ with A Priori Knowledge                ####
// ####                                                                  ####
// #### ================================================================ ####
// ####                                                                  ####
// ####                                                                  ####
// #### Trace Utilizer                                                   ####
// ####                                                                  ####
// #### Version 1.00  --  October 10, 2000                               ####
// ####                                                                  ####
// #### Copyright (C) 2000 Thomas Dreibholz                              ####
// #### University of Bonn, Department of Computer Science IV            ####
// #### EMail: Dreibholz@bigfoot.com                                     ####
// #### WWW:   http://www.bigfoot.com/~dreibholz/diplom/index.html       ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "tdtf.h"
#include "tracearray.h"
#include "mpegtracearray.h"
#include "tdtfwriter.h"
#include "tdtfmediareader.h"
#include "globaltraceconfiguration.h"
#include "mpegwriterqosdescription.h"


using namespace Coral;



// ###### Update all intervals of given frame rate ##########################
void update(TDTFMediaReader&          trace,
            TraceQoSDescription& qosDescription,
            const cardinal            bwThreshold,
            const double              utThreshold)
{
   // ====== Count frame rates ==============================================
   cardinal frameRates = 0;
   double fr = trace.getMinFrameRate();
   while(fr != trace.getMaxFrameRate()) {
      fr = trace.getNextFrameRateForRate(fr);
      frameRates++;
   }
   frameRates++;


   // ====== Count intervals ================================================
   cardinal intervals[frameRates];
   cardinal frames[frameRates];
   double   frameRate[frameRates];
   fr = trace.getMinFrameRate();
   for(cardinal i = 0;i < frameRates;i++) {
      cardinal position  = 0;
      cardinal length    = 0;
      intervals[i] = 0;
      frames[i]    = trace.getFrames(fr);
      frameRate[i] = fr;
      do {
         // ====== Go to next interval ======================================
         const IntervalHeader* ih = trace.getIntervalHeader(position,fr);
         if(ih == NULL) {
            cerr << "ERROR: update() - Interval header not found!" << endl;
            exit(1);
         }
         length = translate32(ih->Length);
         position += length;
         intervals[i]++;
      } while(position < frames[i]);
      fr = trace.getNextFrameRateForRate(fr);
   }
   const cardinal maxRateIndex = frameRates - 1;

   for(cardinal fposition = 0;fposition < frames[maxRateIndex];fposition++) {
      const card64 position = trace.framePositionToPosition(frameRate[maxRateIndex],fposition);
      if(position >= trace.getMaxPosition()) {
         break;
      }

printf("F%04d\n",fposition);

      qosDescription.initTraceDescription(
         &trace, position,
         trace.getMaxPosition(),
         frameRate[maxRateIndex]);

      trace.setPosition(position);
      trace.setFrameRate(25.0);

      qosDescription.updateDescription(0,1024);


      // ====== Calculate resource/utilization list ======================
      ResourceUtilizationPoint rupList[TraceConfig.RUPoints];
      const cardinal newEntries =
         qosDescription.calculateResourceUtilizationList(
            (ResourceUtilizationPoint*)&rupList,
            TraceConfig.BandwidthThreshold,
            TraceConfig.UtilizationThreshold,
            TraceConfig.RUPoints);
      ResourceUtilizationPoint::sortResourceUtilizationList(
                                   (ResourceUtilizationPoint*)&rupList,
                                   0, newEntries - 1);


      // ====== Verify result ===============================================
      ResourceUtilizationPoint pcRupList[newEntries];
      const cardinal pcEntries = qosDescription.getPrecomputedResourceUtilizationList(
            (ResourceUtilizationPoint*)&pcRupList,
            TraceConfig.BandwidthThreshold, TraceConfig.UtilizationThreshold, newEntries);
      if(pcEntries != newEntries) {
         cerr << "INTERAL ERROR: update() - Differing entry counts!" << endl;

         for(cardinal i = 0;i < pcEntries;i++) {
            cout << pcRupList[i] << " PC" << endl;
            cout << rupList[i]   << " ORIG" << endl;
            if(rupList[i] != pcRupList[i]) {
               cerr << "INTERAL ERROR: update() - Differing RU lists!" << endl;
               exit(1);
            }
         }
      }

      for(cardinal i = 0;i < pcEntries;i++) {
         const double f = rupList[i].FrameRate;
         const IntervalHeader* ih = trace.getIntervalHeader(
            trace.positionToFramePosition(f,position),f);
         cardinal s = translate32(ih->Position);
         cardinal l = translate32(ih->Length);
         for(cardinal j = s;j < s + l;j++) {
            const card64 p = trace.framePositionToPosition(f,j);
//            if(j == s) printf("S=%d  P=%Ld   FR=%f\n",s,p,f);

            const cardinal layers = qosDescription.getLayers();
            for(cardinal k = 0;k < layers;k++) {
               cardinal m = trace.getTraceFrameSize(p,k,f);
               cardinal n = m;


         AbstractLayerDescription* ald = qosDescription.getLayer(k);
         const double alpha =
            ald->getFrameSizeScaleFactorForSize(
               f,
               AbstractLayerDescription::bandwidthToFrameSize(
                  f,rupList[i].LayerBandwidthInfo[k].BytesPerSecond));
         const double min = (double)ald->getMinFrameSize(f);
         const double max = (double)ald->getMaxFrameSize(f);
         const double ManagerScaleFactor    = (min + alpha * (max - min)) / max;
         const double ManagerFrameSizeLimit = (cardinal)floor(max * ManagerScaleFactor);

         m = (cardinal)floor((double)m * ManagerScaleFactor);

/*
// Minimum darf unterschritten werden, es ist je ein Minimum des
// EE-Constraints und gilt als *Bandbreite-Limit*, aber *nicht* als FrameSize-Limit!!!!
         if(m < min) {
            printf("%d => %d   MIN=%f  MAX=%f\n",n,m,min,max);
            puts("ERROR #1");
            exit(0);
         }
*/
//         if((m==max) && (m>0)) printf("%d == %f\n",m,max);

         if(m > max) {
            puts("ERROR #2");
            exit(0);
         }
         if((m == 0) && (n > 0)) {
                  printf("P=%d Y=%Ld FR=%f\n",j,p,f);
                  printf("%d <= %f\n",m,ManagerFrameSizeLimit);

            puts("ERROR #3: Scale down to 0");
            exit(0);
         }
         if(m > ManagerFrameSizeLimit) {
                  printf("P=%d Y=%Ld FR=%f\n",j,p,f);
                  printf("%d <= %f\n",m,ManagerFrameSizeLimit);

            puts("ERROR #4: Limit exceeded");
            exit(0);
         }

            }
         }
      }
   }
}



// ###### Main program ######################################################
int main(int argc, char* argv[])
{
   // ===== Check arguments =================================================
   if(argc < 3) {
      cerr << "Usage: " << argv[0] << endl
           << " [Util?????] [Trace File]" << endl;
      exit(0);
   }
   cardinal bwThreshold = 10000;
   double utThreshold   = 0.10;
   for(cardinal i = 2;i < (cardinal)argc;i++) {
      if(!(strncasecmp(argv[i],"-bwthreshold=",13)))      bwThreshold = atol(&argv[i][13]);
      else if(!(strncasecmp(argv[i],"-utthreshold=",13))) utThreshold = atof(&argv[i][13]);
      else if(argv[i][0] == '-') {
         cerr << "Wrong parameter: " << argv[i] << endl;
         exit(1);
      }
   }
   if(bwThreshold < 1) {
      bwThreshold = 1;
   }
   if(utThreshold < 0.0) {
      utThreshold = 0.0;
   }
   else if(utThreshold > 1.0) {
      utThreshold = 1.0;
   }


   // ====== Open TDTF file =================================================
   if(TraceConfig.load(argv[1],true) == false) {
      cerr << "ERROR: Unable to load configuration!" << endl;
      exit(1);
   }


   TDTFMediaReader trace;
   if(trace.open(argv[2],true)) {
      // ====== Initialize MPEGQoSDescription ===============================
      MPEGWriterQoSDescription qosDescription(&TraceConfig);
      qosDescription.initTraceDescription(
         &trace, trace.getPosition(),
         trace.getMaxPosition(), trace.getMaxFrameRate());


      // ====== Print information ===========================================
      const TDTFPrefix* tp = trace.getTDTFPrefix();
      cout << "TDTF Utilizer  - Copyright (C) 2000 Thomas Dreibholz" << endl
           << "----------------------------------------------------" << endl
           << endl;
      cout << "Input File:" << endl;
      cout << "   Title     = " << tp->Title     << endl;
      cout << "   Copyright = " << tp->Copyright << endl;
      cout << "   Comment   = " << tp->Comment   << endl;
      cout << "   URL       = " << tp->URL       << endl;
      cout << endl;
      cout << "Bandwidth Threshold   = " << bwThreshold << endl;
      cout << "Utilization Threshold = " << utThreshold << endl;
      cout << endl;


      // ====== Update all intervals ========================================
      update(trace,qosDescription,bwThreshold,utThreshold);
   }
   else {
      cerr << "ERROR: Unable to load TDTF trace file <" << argv[2] << ">!" << endl;
      exit(1);
   }

   return(0);
}
