// ##########################################################################
// ####                                                                  ####
// ####                    Master Thesis Implementation                  ####
// ####  Management of Layered Variable Bitrate Multimedia Streams over  ####
// ####                  DiffServ with A Priori Knowledge                ####
// ####                                                                  ####
// #### ================================================================ ####
// ####                                                                  ####
// ####                                                                  ####
// #### Trace Generator                                                  ####
// ####                                                                  ####
// #### Version 1.00  --  October 10, 2000                               ####
// ####                                                                  ####
// #### Copyright (C) 2000 Thomas Dreibholz                              ####
// #### University of Bonn, Department of Computer Science IV            ####
// #### EMail: Dreibholz@bigfoot.com                                     ####
// #### WWW:   http://www.bigfoot.com/~dreibholz/diplom/index.html       ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "tdtf.h"
#include "mpegtracearray.h"
#include "mp3tracearray.h"
#include "globaltraceconfiguration.h"
#include "tdtfwriter.h"


using namespace Coral;


// ###### Main program ######################################################
int main(int argc, char* argv[])
{
   // ===== Check arguments =================================================
   if(argc < 4) {
      cerr << "Usage: " << argv[0] << endl
           << "[Trace Configuration File] [Output File] [MPEG|MP3] {-input=name} {-frames=limit}" << endl;
      exit(0);
   }
   cardinal type = 0;
   if(!(strcasecmp(argv[3],"MPEG")))     type = 1;
   else if(!(strcasecmp(argv[3],"MP3"))) type = 2;
   else {
      cerr << "Wrong type: " << argv[3] << endl;
   }
   char* fileName  = (char*)&TraceConfig.InputName;
   cardinal frames = (cardinal)-1;
   for(cardinal i = 4;i < (cardinal)argc;i++) {
      if(!(strncasecmp(argv[i],"-input=",7)))       fileName = &argv[i][7];
      else if(!(strncasecmp(argv[i],"-frames=",8))) frames   = (cardinal)atol(&argv[i][8]);
      else if(argv[i][0] == '-') {
         cerr << "Wrong parameter: " << argv[i] << endl;
         exit(1);
      }
   }


   // ====== Open input files, load configuration and trace =================
   if(TraceConfig.load(argv[1],false) == false) {
      cerr << "ERROR: Unable to load configuration!" << endl;
      exit(1);
   }
   if(!(strcmp(fileName,""))) {
      cerr << "ERROR: No input file given!" << endl;
      exit(1);
   }
   MPEGTraceArray mpegInput(TraceConfig);
   MP3TraceArray  mp3Input(TraceConfig);
   TraceArray* input = NULL;
   if(type == 1) {
      if(mpegInput.load(fileName,
                      TraceConfig.FramePattern,
                      TraceConfig.FrameRate,
                      TraceConfig.ExtLayers,
                      TraceConfig.FakeE1,
                      TraceConfig.FakeE2) == false) {
         cerr << "Unable to load MPEG trace file <" << fileName << ">!" << endl;
         exit(1);
      }
      input = &mpegInput;
   }
   else if(type == 2) {
      if(mp3Input.load(fileName) == false) {
         cerr << "Unable to load MP3 trace file <" << fileName << ">!" << endl;
         exit(1);
      }
      input = &mp3Input;
   }
   else {
      exit(1);
   }
   TDTFWriter output;


   // ====== Print information ===========================================
   cout << "TDTF Trace Generator - Copyright (C) 2000 Thomas Dreibholz" << endl
        << "----------------------------------------------------------" << endl
        << endl;
   cout << "Configuration File       = " << argv[1]      << endl;
   cout << "Input File               = " << fileName     << endl;
   cout << "Output File              = " << argv[2]      << endl;
   cout << "Frames                   = " << input->Frames;
   if(frames < input->Frames) {
      const cardinal frameRate = (cardinal)rint(TraceConfig.FrameRate);
      input->Frames = max(frames - (frames % frameRate),frameRate);
      cout << ", limited to " << input->Frames;
   }
   cout << endl;
   cout << TraceConfig << endl;
   // cout << *input << endl;


   // ====== Generator output file ==========================================
   output.generate(argv[2],input,TraceConfig);


   return(0);
}
