// ##########################################################################
// ####                                                                  ####
// ####                    Master Thesis Implementation                  ####
// ####  Management of Layered Variable Bitrate Multimedia Streams over  ####
// ####                  DiffServ with A Priori Knowledge                ####
// ####                                                                  ####
// #### ================================================================ ####
// ####                                                                  ####
// ####                                                                  ####
// #### Trace Printer                                                    ####
// ####                                                                  ####
// #### Version 1.00  --  October 10, 2000                               ####
// ####                                                                  ####
// #### Copyright (C) 2000 Thomas Dreibholz                              ####
// #### University of Bonn, Department of Computer Science IV            ####
// #### EMail: Dreibholz@bigfoot.com                                     ####
// #### WWW:   http://www.bigfoot.com/~dreibholz/diplom/index.html       ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "tdtf.h"
#include "tdtfreader.h"


#include <sys/time.h>


using namespace Coral;



// ###### Main program ######################################################
int main(int argc, char* argv[])
{
   // ===== Check arguments =================================================
   if(argc < 2) {
      cerr << "Usage: " << argv[0] << endl
           << "[Trace File] {-rate=rate} {-no-ee} {-full-ee} {-no-rul}" << endl;
      exit(0);
   }
   bool   optFullEE   = false;
   bool   optPrintEE  = true;
   bool   optPrintRUL = true;
   bool   optPrintUC  = true;
   double frameRate  = -HUGE_VAL;
   for(cardinal i = 2;i < (cardinal)argc;i++) {
      if(!(strcasecmp(argv[i],"-no-ee")))         optPrintEE  = false;
      else if(!(strcasecmp(argv[i],"-no-rul")))   optPrintRUL = false;
      else if(!(strcasecmp(argv[i],"-no-uc")))    optPrintUC  = false;
      else if(!(strcasecmp(argv[i],"-full-ee")))  optFullEE   = true;
      else if(!(strncasecmp(argv[i],"-rate=",6))) frameRate   = atof(&argv[i][6]);
      else if(argv[i][0] == '-') {
         cerr << "Wrong parameter: " << argv[i] << endl;
         exit(1);
      }
   }


   // ====== Open TDTF file =================================================
   TDTFReader traceReader;
   if(traceReader.open(argv[1])) {
      const TDTFPrefix* tp = traceReader.getTDTFPrefix();
      const TDTFSuffix* ts = traceReader.getTDTFSuffix();
      char str[256];


      // ====== Print information ===========================================
      cout << "TDTF Trace Printer - Copyright (C) 2000 Thomas Dreibholz" << endl
           << "--------------------------------------------------------" << endl
           << endl;
      cout << "Title            = " << tp->Title     << endl;
      cout << "Copyright        = " << tp->Copyright << endl;
      cout << "Comment          = " << tp->Comment   << endl;
      cout << "URL              = " << tp->URL       << endl;

      cout << "Trace Creation:" << endl;
      time_t timeStamp1  = translate64(tp->TraceTimeStamp) / 1000000;
      struct tm* timeptr = localtime(&timeStamp1);
      strftime((char*)&str,256,"%A, %B %d %G, %T",timeptr);
      cout << "   Generation Start = " << str << endl;

      time_t timeStamp2 = translate64(ts->TraceTimeStamp) / 1000000;
      timeptr   = localtime(&timeStamp2);
      strftime((char*)&str,256,"%A, %B %d %G, %T",timeptr);
      cout << "   Generation End   = " << str << endl;

      time_t diff = timeStamp2 - timeStamp1;
      snprintf((char*)&str,sizeof(str),"%ld:%02ld:%02ld",
                  diff / 3600, (diff / 60) % 60, diff % 60);
      cout << "   Duration         = " << str << endl;

      cout << "Utilization Creation:" << endl;
      timeStamp1 = translate64(tp->UtilizationTimeStamp) / 1000000;
      if(timeStamp1 == 0) {
         optPrintRUL = false;
         optPrintUC  = false;
         cout << "   Utilization has not been created yet!" << endl;
      }
      else {
         timeptr = localtime(&timeStamp1);
         strftime((char*)&str,256,"%A, %B %d %G, %T",timeptr);
         cout << "   Generation Start = " << str << endl;

         timeStamp2 = translate64(ts->UtilizationTimeStamp) / 1000000;
         timeptr    = localtime(&timeStamp2);
         strftime((char*)&str,256,"%A, %B %d %G, %T",timeptr);
         cout << "   Generation End   = " << str << endl;

         diff = timeStamp2 - timeStamp1;
         snprintf((char*)&str,sizeof(str),"%ld:%02ld:%02ld",
                     diff / 3600, (diff / 60) % 60, diff % 60);
         cout << "   Duration         = " << str << endl;
      }

      cout << "Frame Rates      = {";
      double rate = traceReader.getMinFrameRate();
      while(rate != traceReader.getMaxFrameRate()) {
         cout << rate << " ";
         rate = traceReader.getNextFrameRateForRate(rate);
      }
      cout << traceReader.getMaxFrameRate() << "}" << endl;

      cout << endl;


      // ====== Print trace for given frame size ============================
      if(frameRate > 0.0) {
         traceReader.print(cout,frameRate,
                           optPrintEE,!optFullEE,optPrintRUL,optPrintUC);
      }
      else {
         double rate = traceReader.getMinFrameRate();
         while(rate != traceReader.getMaxFrameRate()) {
            traceReader.print(cout,rate,
                              optPrintEE,!optFullEE,optPrintRUL,optPrintUC);
            rate = traceReader.getNextFrameRateForRate(rate);
         }
         traceReader.print(cout,traceReader.getMaxFrameRate(),
                           optPrintEE,!optFullEE,optPrintRUL,optPrintUC);
      }
   }
   else {
      cerr << "ERROR: Unable to load TDTF trace file <" << argv[1] << ">!" << endl;
      exit(1);
   }

   return(0);
}
