// ##########################################################################
// ####                                                                  ####
// ####                    Master Thesis Implementation                  ####
// ####  Management of Layered Variable Bitrate Multimedia Streams over  ####
// ####                  DiffServ with A Priori Knowledge                ####
// ####                                                                  ####
// #### ================================================================ ####
// ####                                                                  ####
// ####                                                                  ####
// #### Utility Function                                                 ####
// ####                                                                  ####
// #### Version 1.00  --  October 10, 2000                               ####
// ####                                                                  ####
// #### Copyright (C) 2000 Thomas Dreibholz                              ####
// #### University of Bonn, Department of Computer Science IV            ####
// #### EMail: Dreibholz@bigfoot.com                                     ####
// #### WWW:   http://www.bigfoot.com/~dreibholz/diplom/index.html       ####
// ####                                                                  ####
// ##########################################################################


#ifndef UTILITYFUNCTION_H
#define UTILITYFUNCTION_H


#include "system.h"
#include "tdtf.h"


namespace Coral {


/**
  * Evaluate utility function of given type.
  *
  * @param type Utility function type.
  * @param scaleFactor Scale factor to evaluate utility function for (out of [0,1]).
  * @param constantArray Array of utility function's constants.
  * @param constants Number of constants.
  * @return Result.
  */
double evaluateUtilityFunction(const cardinal type,
                               const double   scaleFactor,
                               const double*  constantArray,
                               const cardinal constants);


/**
  * Evaluate utility function of given type using byte-order translated
  * constants (translateToDouble()).
  *
  * @param type Utility function type.
  * @param scaleFactor Scale factor to evaluate utility function for (out of [0,1]).
  * @param constantArray Array of utility function's constants.
  * @param constants Number of constants.
  * @return Result.
  *
  * @see translateToDouble
  * @see translateToBinary
  */
inline double evaluateUtilityFunctionTranslated(const cardinal type,
                                                const double   scaleFactor,
                                                const card64*  constantArray,
                                                const cardinal constants);


}


#include "utilityfunction.icc"


#endif
